﻿using Hotline.Application.FlowEngine;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.Workflows;
using Hotline.Orders;
using Hotline.Settings;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.Order;
using MediatR;
using XF.Domain.Entities;
using XF.Domain.Exceptions;

namespace Hotline.Application.Handlers.FlowEngine;

public class NextStepHandler : INotificationHandler<NextStepNotify>
{
    private readonly IWorkflowApplication _workflowApplication;
    private readonly IOrderDomainService _orderDomainService;

    public NextStepHandler(
        IWorkflowApplication workflowApplication,
        IOrderDomainService orderDomainService)
    {
        _workflowApplication = workflowApplication;
        _orderDomainService = orderDomainService;
    }

    /// <summary>Handles a notification</summary>
    /// <param name="notification">The notification</param>
    /// <param name="cancellationToken">Cancellation token</param>
    public async Task Handle(NextStepNotify notification, CancellationToken cancellationToken)
    {
        var workflow = notification.Workflow;
        var data = notification.Dto;
        //var assignMode = await _workflowApplication.GetFlowAssignModeAsync(notification.StepDefine,
        //    data.NextHandlers.Select(d => d.Id).ToList(), cancellationToken);

        switch (workflow.ModuleCode)
        {
            case WorkflowModuleConsts.OrderManage:
                await _orderDomainService.ManageFlowNextAsync(
                    notification.FlowAssignMode, notification.IsCountersignEnd, notification.IsCountersignStart,
                    workflow.Id, workflow.CurrentStepTime, workflow.CurrentStepName, workflow.ExpiredTime,
                    cancellationToken);
                break;
        }
    }
}