﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Application.FlowEngine;
using Hotline.FlowEngine.Notifications;
using Hotline.Orders;
using Hotline.Settings;
using Hotline.Share.Enums.Order;
using MediatR;
using XF.Domain.Exceptions;

namespace Hotline.Application.Handlers.FlowEngine
{
    public class StartWorkflowHandler : INotificationHandler<StartWorkflowNotify>
    {
        private readonly IOrderDomainService _orderDomainService;

        public StartWorkflowHandler(
            IOrderDomainService orderDomainService)
        {
            _orderDomainService = orderDomainService;
        }

        /// <summary>Handles a notification</summary>
        /// <param name="notification">The notification</param>
        /// <param name="cancellationToken">Cancellation token</param>
        public async Task Handle(StartWorkflowNotify notification, CancellationToken cancellationToken)
        {
            var workflow = notification.Workflow;
            var data = notification.Dto;

            switch (workflow.ModuleCode)
            {
                case WorkflowModuleConsts.OrderManage:
                    await _orderDomainService.ManageFlowNextAsync(notification.FlowAssignMode,
                        false, notification.IsCountersignStart,
                        workflow.Id, workflow.CurrentStepTime, workflow.CurrentStepName, workflow.ExpiredTime, cancellationToken);
                    break;
            }
        }
    }
}
