﻿using Hotline.KnowledgeBase;
using Hotline.Repository.SqlSugar.DataPermissions;
using SqlSugar;
using XF.Domain.Dependency;

namespace Hotline.Repository.SqlSugar.Knowledge
{
    public class KnowledgeTypeRepository : BaseRepository<KnowledgeType>, IKnowledgeTypeRepository, IScopeDependency
    {
        public KnowledgeTypeRepository(ISugarUnitOfWork<HotlineDbContext> uow, IDataPermissionFilterBuilder dataPermissionFilterBuilder) : base(uow, dataPermissionFilterBuilder)
        {

        }

        /// <summary>
        /// 查询树形
        /// </summary>
        /// <param name="Id"></param>
        /// <param name="IsEnable"></param>
        /// <returns></returns>
        public async Task<List<KnowledgeType>> GetTree(string Id, string? IsEnable)
        {
            return await Db.Queryable<KnowledgeType>().WhereIF(!string.IsNullOrEmpty(IsEnable), p=>p.IsEnable==true).ToTreeAsync(it => it.Child, it => it.ParentId, Id);
        }

        /// <summary>
        /// 获取子级
        /// </summary>
        /// <param name=""></param>
        /// <returns></returns>
        public async Task<List<KnowledgeType>> GetChild(string Id)
        {
            return await Db.Queryable<KnowledgeType>().ToChildListAsync(it => it.ParentId, Id);
        }

        /// <summary>
        /// 获取父级
        /// </summary>
        /// <param name=""></param>
        /// <returns></returns>
        public async Task<List<KnowledgeType>> GetParent(string Id)
        {
            return await Db.Queryable<KnowledgeType>().ToParentListAsync(it => it.ParentId, Id);
        }
    }

}
