﻿using Hotline.FlowEngine.Workflows;
using Hotline.Share.Enums.Order;
using Hotline.Users;
using SqlSugar;
using XF.Domain.Extensions;
using XF.Domain.Repository;

namespace Hotline.Orders
{
    /// <summary>
    /// 工单
    /// </summary>
    public partial class Order : PositionWorkflowEntity
    {
        #region 来电信息

        /// <summary>
        /// 来源渠道
        /// </summary>
        public EChannel Channel { get; set; }

        /// <summary>
        /// 接线员工(userId)
        /// </summary>
        public string EmployeeId { get; set; }

        /// <summary>
        /// 来电号码
        /// </summary>
        [SugarColumn(IsNullable = true)]
        public string? FromPhone { get; set; }

        /// <summary>
        /// 转接号码（转接来源）
        /// </summary>
        [SugarColumn(IsNullable = true)]
        public string? TransferPhone { get; set; }

        /// <summary>
        /// 来电/信人姓名
        /// </summary>
        public string FromName { get; set; }

        /// <summary>
        /// 来电/信人性别
        /// </summary>
        public EGender FromGender { get; set; }

        /// <summary>
        /// 来电/信人身份
        /// </summary>
        public EIdentityType IdentityType { get; set; }

        /// <summary>
        /// 证件类型
        /// </summary>
        [SugarColumn(IsNullable = true)]
        public string? LicenceTypeCode { get; set; }

        [SugarColumn(IsNullable = true)]
        public string? LicenceType { get; set; }

        /// <summary>
        /// 证件号码
        /// </summary>
        [SugarColumn(IsNullable = true)]
        public string? LicenceNo { get; set; }

        /// <summary>
        /// 年龄段
        /// </summary>
        [SugarColumn(IsNullable = true)]
        public string? AgeRangeCode { get; set; }

        [SugarColumn(IsNullable = true)]
        public string? AgeRange { get; set; }

        /// <summary>
        /// 联系电话
        /// </summary>
        [SugarColumn(ColumnDescription = "联系电话", IsNullable = true)]
        public string? Contact { get; set; }

        [SugarColumn(ColumnDescription = "联系电话脱敏", IsNullable = true)]
        public string? ContactMask { get; set; }

        /// <summary>
        /// 是否接受短信，勾选校验手机号
        /// </summary>
        public bool AcceptSms { get; set; }

        /// <summary>
        /// 是否需要联系
        /// </summary>
        public bool NeedContact { get; set; }

        /// <summary>
        /// 工作单位(当“来电/信人身份”为“企业”时必填，其他情况非必填)
        /// </summary>
        [SugarColumn(ColumnDescription = "工作单位", IsNullable = true)]
        public string? Company { get; set; }

        #endregion

        #region 诉求信息

        /// <summary>
        /// 工单编码（202201010001）
        /// </summary>
        public string No { get; set; }

        /// <summary>
        /// 工单类型
        /// </summary>
        public EOrderType OrderType { get; set; }

        /// <summary>
        /// 受理类型
        /// </summary>
        public EAcceptType AcceptType { get; set; }

        /// <summary>
        /// 紧急程度
        /// </summary>
        public EEmergencyLevel EmergencyLevel { get; set; } = EEmergencyLevel.Normal;

        public string Title { get; set; }

        /// <summary>
        /// 热点
        /// </summary>
        public string HotspotId { get; set; }
        public string Hotspot { get; set; }
        public string HotspotSpliceName { get; set; }

        /// <summary>
        /// 外部数据（为前端提供级联功能）
        /// </summary>
        [SugarColumn(IsNullable = true)]
        public string? HotspotExternal { get; set; }

        /// <summary>
        /// 事发时间
        /// </summary>
        public DateTime? IncidentTime { get; set; }

        /// <summary>
        /// 重复工单
        /// </summary>
        [SugarColumn(ColumnDescription = "重复工单Id", IsNullable = true)]
        public string? DuplicateId { get; set; }

        [SugarColumn(ColumnDescription = "重复工单标题", IsNullable = true)]
        public string? DuplicateTitle { get; set; }

        /// <summary>
        /// 推送分类
        /// </summary>
        [SugarColumn(IsNullable = true)]
        public string? PushTypeCode { get; set; }

        [SugarColumn(IsNullable = true)]
        public string? PushType { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        [SugarColumn(ColumnDataType = "varchar(1000)", IsJson = true)]
        public List<string> Additions { get; set; } = new();

        /// <summary>
        /// 诉求内容
        /// </summary>
        [SugarColumn(ColumnDataType = "varchar(2000)")]
        public string Content { get; set; }

        #endregion

        #region 工单属性

        /// <summary>
        /// 当前节点名称
        /// </summary>
        [SugarColumn(ColumnDescription = "当前节点名称", IsNullable = true)]
        public string? CurrentStepName { get; set; }

        /// <summary>
        /// 到达当前节点时间
        /// </summary>
        public DateTime? CurrentStepTime { get; set; }

        /// <summary>
        /// 工单状态
        /// </summary>
        public EOrderStatus Status { get; set; }

        /// <summary>
        /// 开始时间
        /// </summary>
        public DateTime? StartTime { get; set; }

        /// <summary>
        /// 过期时间
        /// </summary>
        public DateTime? ExpiredTime { get; set; }

        /// <summary>
        /// 工单办理完成时间，实际办理部门的办理时间（汇总节点的上一个非汇总节点办理完成时间，开启会签时清空该字段）
        /// </summary>
        public DateTime? CompleteTime { get; set; }

        /// <summary>
        /// 归档时间（暂为流程结束时间，因流程结束自动归档）
        /// </summary>
        public DateTime? FiledTime { get; set; }

        /// <summary>
        /// 过期状态 //todo 延迟消息更新此字段
        /// </summary>
        public EExpiredStatus ExpiredStatus { get; set; }

        #endregion

        /// <summary>
        /// 接线员(服务人)
        /// </summary>
        [Navigate(NavigateType.OneToOne, nameof(EmployeeId))]
        public User Employee { get; set; }

        [Navigate(NavigateType.OneToOne, nameof(WorkflowId))]
        public Workflow Workflow { get; set; }
    }

    /// <summary>
    /// 渠道为电话时存在字段
    /// </summary>
    public partial class Order
    {
        /// <summary>
        /// 工单扩展信息（12315-投诉）
        /// </summary>
        [Navigate(NavigateType.OneToOne, nameof(Id))]
        public OrderComplain OrderComplain { get; set; }

        /// <summary>
        /// 工单扩展信息（12315-举报）
        /// </summary>
        [Navigate(NavigateType.OneToOne, nameof(Id))]
        public OrderReport OrderReport { get; set; }

        #region Method

        public void Init(string employeeId)
        {
            EmployeeId = employeeId;
            if (!string.IsNullOrEmpty(Contact))
                ContactMask = Contact.MaskPhoneNumber();
            Status = EOrderStatus.Temporary;
            ExpiredStatus = EExpiredStatus.Normal;
            StartTime = DateTime.Now;
        }

        /// <summary>
        /// 开始工单办理流程
        /// </summary>
        /// <param name="workflowId"></param>
        public void StartManageFlow(string workflowId)
        {
            WorkflowId = workflowId;
            Status = EOrderStatus.WaitForSign;
        }

        /// <summary>
        /// 接办
        /// </summary>
        public void Accept()
        {
            Status = EOrderStatus.Handling;
        }

        /// <summary>
        /// 工单办理流程流转（每次办理都会调用该方法）
        /// </summary>
        public void ManageFlow(
            bool isCountersignStart,
            bool isCountersignEnd, 
            DateTime? currentStepTime, 
            string? currentStepName, 
            DateTime expiredTime)
        {
            //1.如果order未处于会签中，则判断是否发起会签（isstartCountersign） 2.如果处于会签中，则判断会签是否结束（isCountersignEnd）
            if (Status is EOrderStatus.Countersigning && isCountersignEnd)
            {
                Status = EOrderStatus.WaitForSign;
            }
            else if (Status is not EOrderStatus.Countersigning && isCountersignStart)
            {
                Status = EOrderStatus.Countersigning;
            }

            CurrentStepTime = currentStepTime;
            CurrentStepName = currentStepName;
            ExpiredTime = expiredTime;
            Status = EOrderStatus.WaitForSign;
        }

        /// <summary>
        /// 最终办理
        /// </summary>
        public void FinalManage()
        {
            Status = EOrderStatus.Completed;
            CompleteTime = DateTime.Now;
        }

        /// <summary>
        /// 撤回最终办理
        /// </summary>
        public void RecallFinalManage()
        {
            Status = EOrderStatus.WaitForSign;
            CompleteTime = null;
        }

        /// <summary>
        /// 归档
        /// </summary>
        public void Filed()
        {
            Status = EOrderStatus.Filed;
            FiledTime = DateTime.Now;
        }

        #endregion
    }
}
